/*
 * BridgeQueue.cpp
 *
 *  Created on: Dec 20, 2012
 *      Author: tuleu
 */

#include "BridgeQueue.h"

#include <biorob-cpp/log/Logger.h>

namespace rtio {
namespace phidgets {

BridgeQueue::BridgeQueue(rtio::Manager & manager,
                         rtio::Queue::ID id,
                         int priority,
                         rtio::Queue::Mode m)
	: rtio::Queue(manager,id,priority,m){
}

BridgeQueue::~BridgeQueue(){
}

void BridgeQueue::LoadFromConfig(BridgesSection& bridgesConfig){
	for(BridgesSection::ListOfSubsection::const_iterator b = bridgesConfig.SubSections().begin();
	    b != bridgesConfig.SubSections().end();
	    ++b){
		log(info,"Found phidget device definition '",b->first,"'");
		AddBridgeDevice(b->second);
	}
}

void BridgeQueue::PreSend(){
}

void BridgeQueue::PreRead(){
	for(SetOfBridges::const_iterator b = d_devices.begin();
	    b != d_devices.end();
	    ++b){
		(*b)->StartFullRefresh();
	}

	for(SetOfBridges::const_iterator b = d_devices.begin();
		    b != d_devices.end();
		    ++b){
		(*b)->WaitForFullRefresh();
	}
}

void BridgeQueue::PostRead(){
}


BridgeInput::BridgeInput(const BridgePtr& bridge ,
                         unsigned int input)
	: d_bridge(bridge)
	, d_input(input){
}

BridgeInput::~BridgeInput(){}

void BridgeInput::Read(){
	SetValue(d_bridge->Data(d_input));
}

void BridgeQueue::AddBridgeDevice(std::tr1::shared_ptr<BridgeSection> bridgeConfig){
	BridgePtr b(new Bridge(bridgeConfig->SerialNumber(),10000));
	d_devices.insert(b);
	for(BridgeSection::ListOfSubsection::const_iterator i = bridgeConfig->SubSections().begin();
	    i != bridgeConfig->SubSections().end();
	    ++i){
		AddBridgeInput(i->first,i->second,b);
	}

	b->SetDataRate(bridgeConfig->DataRate());


}

void BridgeQueue::AddBridgeInput(const std::string & name,
                                 std::tr1::shared_ptr<BridgeInputSection> bridgeInputConfig,
                                 BridgePtr bridge){
	unsigned int in(bridgeInputConfig->Input());
	bridge->EnableInput(in,
	                    Bridge::GainFromInt(bridgeInputConfig->Gain()));

	this->RegisterInput(name,rtio::Input::Ptr(new BridgeInput(bridge,in)));
	log(info,"Adds phidget bridget input '",name,"' from device input ",in);

}

} /* namespace phidgets */

} /* namespace rtio */
