/**
 * \file RbioModule.cpp
 *
 *  \date Dec 14, 2012
 *  \author Alexandre Tuleu
 */

#include "RbioModule.h"
#include "ServoQueue.h"
#include <biorob-rbio/servo/ServoConfig.h>

namespace rtio {

	RbioModule::RbioModule(){
	}

	RbioModule::~RbioModule(){
	}

	void RbioModule::InitIO(Manager & manager){
		//create a queue for servos
		Queue::Ptr q = manager.AddQueue<rtio::ServoQueue>(40,Queue::NonRealtime);

		//get all servo with their calibration from biorob-rbio
		ServoGenerator sg;
		sg.LoadAllFiles();

		//servo in the library are updated every period, defined by 20 ns ticks
		static_cast<rtio::ServoQueue*>(q.get())->SetUpdatePeriod(20.0e-9 * sg.Servos().PulsePeriod());
		ServoGenerator::ServoByName servos;
		sg.CreateServoFromDefinition(servos);

		//finally register all of them as output.
		for(ServoGenerator::ServoByName::const_iterator s = servos.begin();
		    s != servos.end();
		    ++s){
			q->RegisterOutput(s->first,
			                  rtio::Output::Ptr(new rtio::ServoQueue::Output(s->second)));
		}
		//done
	}

} /* namespace rx */
