/**
 * \file ServoQueue.cpp
 *
 *  \date Dec 14, 2012
 *  \author Alexandre Tuleu
 */

#include "ServoQueue.h"

#include <native/task.h>
#include <native/timer.h>
#include <sys/io.h>
#include <stdexcept>


namespace rtio {

	ServoQueue::ServoQueue(rtio::Manager& manager,
	        	           rtio::Queue::ID id,
	        	           int priority,
	        	           rtio::Queue::Mode m)
		: rtio::Queue(manager,id,priority,m)
		, d_updatePeriod(0){

	}

	ServoQueue::~ServoQueue(){
	}

	void ServoQueue::SetUpdatePeriod(double second){
		d_updatePeriod = second * 1e9;
	}
	void ServoQueue::PreSend(){		
		static bool once = true;
		if (once) {
			if( iopl(3) != 0 ) {
				throw std::runtime_error(std::string("Could not rise iopl(3) : ") + strerror(errno) );
			}
			once = false;
		}

		d_start = rt_timer_read();
		rt_task_set_mode(T_PRIMARY,0,NULL);

	}
	void ServoQueue::PostRead(){
		rt_task_sleep_until(d_start + d_updatePeriod );
	}

	void ServoQueue::PreRead(){}

	ServoQueue::Output::Output(Servo::Ptr servo)
		: rtio::Output()
		, d_servo(servo){
		SetValue(0.0);
	}

	ServoQueue::Output::~Output(){
		if( iopl(3) != 0 ) {
			throw std::runtime_error(std::string("Could not rise iopl(3) : ") + strerror(errno) );
		}
	}

	void ServoQueue::Output::Send(){
		d_servo->SetCommand(Value());
	}

} /* namespace rbio */
