/**
 * \file Config.cpp
 *
 *  \date Dec 17, 2012
 *  \author Alexandre Tuleu
 */

#include "Config.h"

namespace rtio {
namespace sbcp {

BrushlessMotorSection::BrushlessMotorSection(options::Section::Ptr s)
	: options::ConfigSection(s)
	, d_hPGain(Section())
	, d_hIGain(Section())
	, d_hDGain(Section())
	, d_hStiffness(Section())
	, d_hDamping(Section())
	, d_hPreCompression(Section())
	, d_hMaxSpeed(Section())
	, d_hMaxAcceleration(Section())
	, d_hMaxTorque(Section())
	, d_hRefreshRate(Section())
	, d_hControlMode(Section()){
	InitControlModes();
}


BrushlessMotorSection::~BrushlessMotorSection(){
}

BrushlessMotorSection::ControlModeByName BrushlessMotorSection::s_controlModes;

void BrushlessMotorSection::InitControlModes(){
	if(!s_controlModes.empty()){
		return;
	}
	typedef ::sbcp::amarsi::MotorDriver::Motor Motor;
	s_controlModes["coast"]           = Motor::COAST;
	s_controlModes["position"]        = Motor::POSITION;
	s_controlModes["smooth_position"] = Motor::SMOOTH_POSITION;
	s_controlModes["velocity"]        = Motor::VELOCITY;
	s_controlModes["torque"]          = Motor::TORQUE;

}

BrushlessMotorSection::MotorControlMode BrushlessMotorSection::GetControlMode() const{
	ControlModeByName::const_iterator fi = s_controlModes.find(ControlMode());
	if(fi == s_controlModes.end()){
		throw std::out_of_range("Control mode '" + ControlMode() + "' is unknown");
	}
	return fi->second;
}

BrushlessTemplates::BrushlessTemplates(options::Section::Ptr s)
	: options::DynamicConfigSection<BrushlessMotorSection>(s){
	Section().AddSubSection("hip","hip template parameters");
	Section().AddSubSection("knee","knee template parameters");
}

BrushlessTemplates::~BrushlessTemplates(){}

MotorDriverSection::MotorDriverSection(options::Section::Ptr s)
	: options::ConfigSection(s)
	, d_hBoardID(Section())
	, d_hM1Suffix(Section())
	, d_hM2Suffix(Section())
	, d_hM1Template(Section())
	, d_hM2Template(Section()){
}

MotorDriverSection::~MotorDriverSection(){
}

MotorDriverBoards::MotorDriverBoards(options::Section::Ptr s)
	: options::DynamicConfigSection<MotorDriverSection>(s)
	, d_hBus(Section()){
}

MotorDriverBoards::~MotorDriverBoards(){}


Config::Config()
	: options::Config("rtio-sbcp")
	, d_shBrushless(Section())
	, d_shBoards(Section()){
}

Config::~Config(){
}

} /* namespace sbcp */
} /* namespace rx */
