/**
 * \file Config.h
 *
 *  \date Dec 17, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_RTIO_SBCP_CONFIG_H_
#define ROBO_XENO_RTIO_SBCP_CONFIG_H_

#include <stdint.h>
#include <biorob-cpp/options/Config.h>


#include <libsbcp/devices/amarsi/MotorDriver.h>

/**
 *
 */
namespace rtio {
namespace sbcp {

class BrushlessMotorSection : public options::ConfigSection{
public :
	typedef ::sbcp::amarsi::MotorDriver::Motor::ControlMode MotorControlMode;

	BrushlessMotorSection(options::Section::Ptr s);
	~BrushlessMotorSection();

	BCPP_ADD_OPTION(PGain,"p-gain",uint16_t,400,"P-gain of the motor")
	BCPP_ADD_OPTION(IGain,"i-gain",uint16_t,20 ,"I-gain of the motor")
	BCPP_ADD_OPTION(DGain,"d-gain",uint16_t,200,"D-gain of the motor")

	BCPP_ADD_OPTION(Stiffness,"stiffness",uint16_t,0,"virtual compliance stiffness")
	BCPP_ADD_OPTION(Damping,"damping",uint16_t,0,"virtual compliance damping")
	BCPP_ADD_OPTION(PreCompression,"precompression",uint16_t,1100,"virtual compliance precompression")


	BCPP_ADD_OPTION(MaxSpeed,"max-speed",uint16_t,32000,"Maximal speed")
	BCPP_ADD_OPTION(MaxAcceleration,"max-acceleration",uint16_t,32000,"Maximal speed")
	BCPP_ADD_OPTION(MaxTorque,"max-torque",uint16_t,1100,"Maximal speed")

	BCPP_ADD_OPTION(RefreshRate,"refresh-rate",uint16_t,10,"refresh rate in ms")

	BCPP_ADD_OPTION(ControlMode,"control-mode",std::string,"smooth_position",
	                "Control mode of the motor (coast,position,smooth_position,velocity,torque")
	MotorControlMode GetControlMode() const;

private :

	typedef  std::map<std::string,MotorControlMode> ControlModeByName;
	static ControlModeByName s_controlModes;

	static void InitControlModes();

};

class BrushlessTemplates : public options::DynamicConfigSection<BrushlessMotorSection> {
public :
	BrushlessTemplates(options::Section::Ptr s);
	~BrushlessTemplates();
};

class MotorDriverSection : public options::ConfigSection {
public :
	MotorDriverSection(options::Section::Ptr s);
	~MotorDriverSection();

	BCPP_ADD_OPTION(BoardID,"id",int,-1,"Board ID to use")
	BCPP_ADD_OPTION(M1Suffix,"m1-suffix",std::string,"M1","Suffix for M1 motor (input and output)")
	BCPP_ADD_OPTION(M2Suffix,"m2-suffix",std::string,"M2","Suffix for M2 motor (input and output)")
	BCPP_ADD_OPTION(M1Template,"m1-template",std::string,"hip","Template parameter to use with M1 motor")
	BCPP_ADD_OPTION(M2Template,"m2-template",std::string,"knee","Template parameter to use with M2 motor")

};


class MotorDriverBoards : public options::DynamicConfigSection<MotorDriverSection> {
public :
	MotorDriverBoards(options::Section::Ptr s);
	~MotorDriverBoards();

	BCPP_ADD_OPTION(Bus,"bus",std::string,"default","Name of the sbcp bus to use")
};

/**
 *
 */
class Config : public options::Config{
public:
	Config();
	virtual ~Config();



	BCPP_ADD_SUBSECTION(BrushlessTemplates,Brushless,"brushless","Brushless parameters defintions")
	BCPP_ADD_SUBSECTION(MotorDriverBoards,Boards,"boards","boards definitions")
};

} /* namespace sbcp */
} /* namespace rx */
#endif // ROBO_XENO_RX_SBCP_CONFIG_H_
