/**
 * \file IO.h
 *
 *  \date Dec 17, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_RTIO_SBCP_IO_H_
#define ROBO_XENO_RTIO_SBCP_IO_H_

#include <robo-xeno/rtio/Queue.h>
#include <libsbcp/device/RegisterAccessor.h>

namespace rtio {
namespace sbcp {

template <typename T,typename InputType = ::sbcp::Data >
class Input : public rtio::Input {
public :
	Input(::sbcp::RegisterReader<T,InputType> & reader)
		: d_reader(reader){}
	virtual ~Input(){}

	static rtio::Input::Ptr Create(::sbcp::RegisterReader<T,InputType> & reader){
		return rtio::Input::Ptr(new Input(reader));
	}
	void Read(){
		SetValue(d_reader.Get());
	}
private :
	::sbcp::RegisterReader<T,InputType> & d_reader;
};

template <typename T,typename InputType = ::sbcp::Data >
class Output : public rtio::Output {
public :
	Output(::sbcp::RegisterAccessor<T,InputType> & writer)
		: d_writer(writer){}
	virtual ~Output(){}

	static rtio::Output::Ptr Create(::sbcp::RegisterAccessor<T,InputType> & writer){
		return rtio::Output::Ptr(new Output(writer));
	}

	void Send(){
		d_writer.Set(Value());
	}
private :
	::sbcp::RegisterAccessor<T,InputType> & d_writer;
};



} /* namespace sbcp */
} /* namespace rx */


#endif // ROBO_RX_SBCP_XENO_IO_H_
