/**
 * \file Module.cpp
 *
 *  \date Dec 17, 2012
 *  \author Alexandre Tuleu
 */


#include "Config.h"
#include "Module.h"
#include <biorob-cpp/options/Config.h>
#include <libsbcp/utils/Config.h>

#include "MotorQueue.h"

namespace rtio {
namespace sbcp {

Module::Module(){
}

Module::~Module(){
}

void Module::InitIO(rtio::Manager & manager){
	Config config;
	config.LoadAllFiles();
	unsigned int minFrameSize = config.Boards().SubSections().size();

	::sbcp::Config sbcpConfig;
	sbcpConfig.LoadAllFiles();
	std::tr1::shared_ptr< ::sbcp::Bus> b = sbcpConfig.BusDefinitions().Bus(config.Boards().Bus()).OpenBus(32,minFrameSize);

	if(!b){
		throw std::runtime_error("Could not open bus '" + config.Boards().Bus() + "'.");
	}

	std::tr1::shared_ptr<MotorQueue> q =
	    std::tr1::static_pointer_cast<rtio::sbcp::MotorQueue,rtio::Queue >(manager.AddQueue<MotorQueue>(50,Queue::NonRealtime));

	q->SetBus(b);

	const MotorDriverBoards::ListOfSubsection & boards = config.Boards().SubSections();

	for(MotorDriverBoards::ListOfSubsection::const_iterator b = boards.begin();
	    b != boards.end();
	    ++b){
		q->AddBoard(b->first,*(b->second),config.Brushless());
	}


	q->CalibrateMotors();

}

} /* namespace sbcp */
} /* namespace rx */
