/**
 * \file MotorQueue.h
 *
 *  \date Dec 17, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_RTIO_SBCP_MOTORQUEUE_H_
#define ROBO_XENO_RTIO_SBCP_MOTORQUEUE_H_

#include <set>

#include <libsbcp/bus/Bus.h>
#include <libsbcp/devices/amarsi/MotorDriver.h>

#include <robo-xeno/rtio/Queue.h>

#include "Config.h"

namespace rtio {
namespace sbcp {
/**
 *
 */
class MotorQueue : public rtio::Queue{
public:
	MotorQueue(rtio::Manager & manager,
	           rtio::Queue::ID id,
	           int priority,
	           rtio::Queue::Mode mode);
	virtual ~MotorQueue();

	void SetBus(const std::tr1::shared_ptr< ::sbcp::Bus> & b);

	void AddBoard(const std::string & name,
	              const MotorDriverSection & boardConfig,
	              BrushlessTemplates & templates);

	void CalibrateMotors();

	void PreRead();
	void PreSend();
	void PostRead();
private :
	typedef ::sbcp::amarsi::MotorDriver     MdvBoard;
	typedef std::tr1::shared_ptr<MdvBoard>  MdvBoardPtr;
	typedef std::vector<MdvBoardPtr>        ListOfMdvBoard;
	typedef std::map<MdvBoard::Motor*,std::string> SetOfMotor;
	typedef std::map<MdvBoard::Motor*,MdvBoard::Motor::ControlMode> ControlModeByMotor;
	void Close();
	void Init();

	void AddMotor(const std::string & name,
	              MdvBoard::Motor & motor,
	              const BrushlessMotorSection & config);

	void PrepareForLoop();

	void FetchUncalibratedBoard(SetOfMotor & uncalibratedBoards);

	std::tr1::shared_ptr< ::sbcp::Bus> d_bus;

	ListOfMdvBoard d_boards;
	ControlModeByMotor d_controlModes;
};
} /* namespace sbcp */
} /* namespace rx */

#endif // ROBO_XENO_RX_SBCP_MOTORQUEUE_H_
