/**
 * \file Interface.h
 *
 *  \date Dec 10, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_INTERFACE_H_
#define ROBO_XENO_INTERFACE_H_

#include <biorob-cli/Interface.h>

#include <robo-xeno/process/Process.h>
#include <robo-xeno/main/ProcessRunner.h>
#include <robo-xeno/rtio/Manager.h>

#include "VariableTable.h"
#include "ManagerDisplayWidget.h"
#include <native/task.h>

#include "color.h"
/**
 *
 */
namespace rx {


	/**
	 *
	 */
	class Interface : public cli::Interface {
		DISABLE_COPY(Interface)
		DISABLE_ASSIGNEMENT(Interface)
	public:
		Interface();
		virtual ~Interface();

		void SetRunner(ProcessRunner & runner);
		void SetProcess(Process::Ptr process);
		void SetManager(rtio::Manager &);

		void Save(const std::string & filename);

		void Select(const std::string & propName,const double & increment);

		void Unselect();

		void Start();

		void Pause();


		void StartTask();


		static void SwitchToSecondary();
	private :
	class Variable : public cli::Variable<double> {
	public :
		typedef std::tr1::shared_ptr<rx::Interface::Variable> Ptr;
		Variable(const std::string & name,
		         const Process::ControlInput & ci,
		         VariableTable<double> & vt,
		         rx::Process & p);
		virtual ~Variable();

		void PostSetValueHook();
	private :
		Process::ControlInput   d_ci;
		VariableTable<double> & d_vt;
		rx::Process           & d_process;
	};

	class IncrementCallback : public cli::KeyCallback {
	public :
		IncrementCallback(Variable::Ptr v, double increment);
		virtual ~IncrementCallback();

		int Call(int key);
	private :
		Variable::Ptr d_variable;
		double        d_increment;
	};

	typedef std::map<std::string,Variable::Ptr> VariableByName;
	typedef std::tr1::shared_ptr<RT_TASK>       RtTaskPtr;
	static const char IncrementKey='+';
	static const char DecrementKey='-';

	static void TaskBody(void * itself);
	void        TaskLoop();
	void Init();
	Process::Ptr    d_process;
	ProcessRunner * d_runner;

	RtTaskPtr                 d_task;
	ManagerDisplayWidget::Ptr d_managerDisplay;
	VariableTable<double>::Ptr        d_variableTable;
	VariableByName d_variables;

};
}
#endif // ROBO_XENO_INTERFACE_H_
