/**
 * \file ManagerDisplayWidget.h
 *
 *  \date Dec 10, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_MANAGERDISPLAYWIDGET_H_
#define ROBO_XENO_MANAGERDISPLAYWIDGET_H_

#include <robo-xeno/rtio/Manager.h>
#include <robo-xeno/NativeHolder.h>

#include <biorob-cli/Widget.h>

/**
 *
 */
namespace rx {
/**
 *
 */
class ManagerDisplayWidget : public cli::Widget {
public:
	class LogQueue : public rtio::LogQueue {
	public :
		LogQueue(ID id);
		virtual ~LogQueue();
		void SetManagerDisplay(ManagerDisplayWidget * md);
		void Init();
		void Process();
	private :
		ManagerDisplayWidget * d_display;
	};
	typedef std::tr1::shared_ptr<ManagerDisplayWidget> Ptr;
	ManagerDisplayWidget(Def & def);
	virtual ~ManagerDisplayWidget();

	void InitContentWith(rtio::Manager & manager);



	void PostponedRefresh();
	void RefreshIfNeeded();

private :
	struct NameAndRow{
		NameAndRow(const std::string & name, int row,unsigned int size);
		std::string name;
		int         row;
	};

	typedef std::map<rtio::Input::Ptr,NameAndRow>  NameAndRowByInput;
	typedef std::map<rtio::Output::Ptr,NameAndRow> NameAndRowByOutput;

	static const std::string Unplugged;
	static const std::string Plugged;
	static const unsigned int ProcessBlockWidth = 5;

	static void CutNameIfNeeded(unsigned int maxSize, std::string & name);
	void DrawCenter();
	void DrawBox(int x, int y, int width, int height);
	void VPrint(int x, int y, const std::string & name);

	void HPrint(int x, int y, const std::string & str);

	void PrintInput(const NameAndRow & nr, double value);
	void PrintOutput(const NameAndRow & nr, double value);

	const unsigned int        ColSize;
	const unsigned int        RowSize;
	NativeHolder<RT_MUTEX>    d_mutex;
	bool                      d_refresh;
};
} //namespace rx
#endif // ROBO_XENO_MANAGERDISPLAYWIDGET_H_
