/**
 * \file Main.cpp
 *
 *  \date Dec 7, 2012
 *  \author Alexandre Tuleu
 */

#include "Main.h"

#include <native/types.h>

#include <sys/mman.h>


namespace rx {
	const std::string Main::EmptyString;
	namespace po= boost::program_options;

	Main::Main(int & argc, char ** argv)
		: d_vm()
		, d_desc("Allowed options"){
#ifndef XENO_EMULATION_USED
		if(mlockall(MCL_CURRENT | MCL_FUTURE)){
			throw std::runtime_error(std::string("Could not lock memory in RAM : ") + strerror(errno));
		}
#endif
		d_desc.add_options()
		    ("help,h", "produce help message")
		    ("config,C", po::value<std::string>(), "config file to append")
		    ("template-config,t",po::value<std::string>(), "print a template config file");
		;

		po::store(po::parse_command_line(argc, argv, d_desc), d_vm);
		po::notify(d_vm);

		}

	Main::~Main(){
	}

	const std::string & Main::AdditionnalConfigFile() const{
		if(d_vm.count("config")){
			return d_vm["config"].as<std::string>();
		}
		return EmptyString;
	}


	Main::ProcessRunnerPtr Main::s_pr;

	void Main::SigHandler(int ){
		if(s_pr){
			s_pr->StopTask();
		}
	}

} /* namespace rx */

/**
 * \mainpage
 *
 * robo-xeno is a framework to help developing robotics application on top of
 * xenomai. Its purpose ist to organize a hierarchy of thread to perform
 * realtime IO, and a main rtio::Process that perform the operation on these
 * I/O, The library provide all the synchronization mechanisms to insure that
 * rtio::Queue,rtio::Input rtio::Output, rtio::Process and rtio::LogQueue
 * always access their data through their callback with no concurency.
 *
 * On top of these mechanism, it provides a suits of module to implements
 * thes I/O operation for the common robotic platform found at biorob.
 *
 * There is a separation in two main domain :
 * - rtio : the domain that abstract all Input and Output operation that
 *          could be performed on the platform
 * - rx   : the domain that presents an abstraction of the Process and control
 *          of that process that would be run.ls
 *
 */

