/**
 * \file Main.h
 *
 *  \date Dec 7, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_MAIN_H_
#define ROBO_XENO_MAIN_H_

//#include <robo-xeno/process/Process.h>
#include <robo-xeno/process/Process.h>
#include <boost/program_options.hpp>

#include <biorob-cpp/options/Config.h>

#include "MainConfigSection.h"
#include "ProcessRunner.h"
#include <robo-xeno/interface/Interface.h>
#include <signal.h>
/**
 *
 */
namespace rx {
/**
 * \todo Add logger using an internal buffer + an rt_pipe with stream,
 *   logger as an abstracted version to send (file, net, display).
 * \todo write it.
 */
class Main {
public:
	Main(int & argc, char ** argv);
	virtual ~Main();

	const std::string & AdditionnalConfigFile() const;

	template <typename ConfigType>
	int Exec(const ConfigType & config, Process::Ptr process);
	template <typename ConfigType>
	void ProcessOptions(const ConfigType & config);

private :

	typedef std::tr1::shared_ptr<rx::ProcessRunner> ProcessRunnerPtr;
	static void SigHandler(int c);
	static ProcessRunnerPtr s_pr;

	boost::program_options::variables_map  d_vm;
	boost::program_options::options_description d_desc;

	static const std::string EmptyString;

	void PrintToFile(options::Section::Ptr s, const std::string & filename);
};



	template <typename ConfigType>
	inline int Main::Exec(const ConfigType & config, Process::Ptr process){

		std::tr1::shared_ptr<rx::Interface> interface(new rx::Interface());

		//install signal handler here ....
		signal(SIGINT,&Main::SigHandler);
		rt_task_shadow(NULL,NULL,0,0);


		s_pr = ProcessRunnerPtr(new ProcessRunner(config.Main()));
		s_pr->SetProcess(process);
		s_pr->SetInterface(interface);

		s_pr->StartTask();

		s_pr->WaitForCompletion();
		interface = std::tr1::shared_ptr<rx::Interface>();
		//at this moment ProcessRunner will be closed, manager killed, and
		//the Queue closed .... hopefully
		return 0;
	}

	template <typename ConfigType>
	inline void Main::ProcessOptions(const ConfigType & config){
		if(d_vm.count("help")){
			std::cout << d_desc << std::endl;
			exit(0);
		}

		if(d_vm.count("template-config")){
			config.Print(d_vm["template-config"].as<std::string>());
			exit(0);
		}

	}


}


#endif // ROBO_XENO_MAIN_H_
