/**
 * \file MainConfigSection.h
 *
 *  \date Dec 7, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_MAINCONFIGSECTION_H_
#define ROBO_XENO_MAINCONFIGSECTION_H_

#include <biorob-cpp/options/Option.h>
#include <biorob-cpp/options/Config.h>
#include <robo-xeno/module/ModulesSection.h>
/**
 *
 */
namespace rx{
/**
 *
 */
class MappingSection :  public options::ConfigSection {
public :
	typedef std::map<std::string,std::string> ProcessIOByRTIO;
	MappingSection(options::Section::Ptr s);
	virtual ~MappingSection();

	void Map(ProcessIOByRTIO & result) const;
private :
	ProcessIOByRTIO d_map;
};

class MappingConfigSection : public options::ConfigSection{
public:
	MappingConfigSection(options::Section::Ptr s);
	virtual ~MappingConfigSection();

	BCPP_ADD_SUBSECTION(MappingSection,Out,"out","out mapping; foo = bar will map actuator 'foo' to process output 'bar");
	BCPP_ADD_SUBSECTION(MappingSection,In,"in","in mapping; foo = bar will map sensor 'foo' to process input 'bar");
};

class MainConfigSection : public options::ConfigSection {
public :
	MainConfigSection(options::Section::Ptr s);
	virtual ~MainConfigSection();


	BCPP_ADD_OPTION(Timestep,"timestep",double,0.01,"timestep")
	BCPP_ADD_OPTION(Priority,"priority",int, 70,"priority of the main task")
	BCPP_ADD_OPTION(IterationPerStep,"iteration-per-step",unsigned int,1,"nb iteration per timestep")
	BCPP_ADD_OPTION(CliFeedbackIteration,"cli-feedback-iteration",int,-1,"cli iteration feedback, -1 to disable >1 to refresh data every timestep")
	BCPP_ADD_OPTION(IOLog,"io-log",std::string,"","log file for writing all I/O data")
	BCPP_ADD_OPTION(QueueSynchronizationLog,"queue-synchro-log",std::string,"","log file for when queue start/finish")

	BCPP_ADD_SUBSECTION(MappingConfigSection,Mapping,"mapping","mapping overrides. Please note that there is automatic override rules, if a nammepd io exists and a process export the name, there will be mapped")
	BCPP_ADD_SUBSECTION(ModulesSection,Modules,"modules","modules to load section")
};

}
#endif // ROBO_XENO_MAINCONFIGSECTION_H_
