/**
 * \file ProcessRunner.h
 *
 *  \date Dec 10, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_PROCESSRUNNER_H_
#define ROBO_XENO_PROCESSRUNNER_H_


#include <vector>


#include <native/task.h>
#include <native/mutex.h>

#include <robo-xeno/process/Process.h>
#include <robo-xeno/rtio/Manager.h>
#include <robo-xeno/NativeHolder.h>
#include <robo-xeno/module/ModuleLoader.h>
#include "MainConfigSection.h"

/**
 *
 */
namespace rx {
	class Interface;
/**
 *
 */
class ProcessRunner {
public:
	typedef std::tr1::shared_ptr<rx::Interface> InterfacePtr;
	ProcessRunner(const MainConfigSection & config);
	virtual ~ProcessRunner();



	void StartTask();
	void StopTask();

	void SetPause(bool on = false);

	void WaitForCompletion();

	void SetProcess(Process::Ptr process);
	void SetInterface(InterfacePtr interface);
private :
	typedef std::vector<rtio::Output::Ptr>      ListOfOutput;
	static void Task(void * itselfPtr);
	void MainInit();
	void MainLoop();

	void MapInput();
	void MapOutput();

	const MainConfigSection &           d_config;
	NativeHolder<RT_TASK>       d_task;
	NativeHolder<RT_MUTEX>      d_mutex;
	ModuleLoader  d_moduleLoader;
	Process::Ptr  d_process;
	rtio::Manager d_manager;
	double        d_period;
	bool          d_onPause;
	InterfacePtr  d_interface;
};


}
#endif // ROBO_XENO_PROCESSRUNNER_H_
