/**
 * \file ModuleLoader.cpp
 *
 *  \date Dec 6, 2012
 *  \author Alexandre Tuleu
 */

#include "ModuleLoader.h"

namespace rx {

	ModuleLoader::ModuleLoader()
		: d_modules(){

	}

	ModuleLoader::~ModuleLoader(){
	}


	void ModuleLoader::LoadModuleFromConfig(const ModulesSection & modules){
		ListOfModuleName critical,optionnal;
		SplitNames(modules.Critical(),critical);
		SplitNames(modules.Optionnal(),optionnal);

		//no error propagation if the module is optionnal;
		try {
			LoadModuleList(optionnal);
		} catch (const std::runtime_error & ){
		}
		LoadModuleList(critical);

	}


	void ModuleLoader::SplitNames(std::string allNames,
	                              ListOfModuleName & moduleNames){
		allNames.erase(std::remove_if(allNames.begin(),allNames.end(),isspace),allNames.end());

		moduleNames.clear();

		if(allNames.empty()){
			return;
		}
		std::istringstream is(allNames);
		std::string name;
		while(std::getline(is,name,';')){
			if(name.empty()){
				continue;
			}
			moduleNames.push_back(name);
		}
	}


	void ModuleLoader::LoadModuleList(const ListOfModuleName & modules){
		typedef PluginLoader<rtio::Module> PL;
		std::ostringstream error;
		PL pl;
		pl.SetHint((PL::LoadingHint)(PL::IsCMakeModuleObject | PL::ResolveAllSymbols));
		for(ListOfModuleName::const_iterator m = modules.begin();
		    m != modules.end();
		    ++m){


			ModuleByName::iterator fi = d_modules.find(*m);
			if(fi != d_modules.end()){
				continue;
			}

			pl.SetFilename(*m);
			ModulePtr module;
			try {
				module = pl.LoadPlugin();
			} catch (const std::exception & e){
				error << e.what() << std::endl;
			}
			d_modules.insert(std::make_pair(*m,module));
		}
		if(!error.str().empty()){
			throw std::runtime_error(error.str());
		}
	}

	void ModuleLoader::StartQueuesOfModules(rtio::Manager & manager){
		for(ModuleByName::iterator m = d_modules.begin();
		    m != d_modules.end();
		    ++m){
			m->second->InitIO(manager);
		}
	}

} /* namespace rx */
