/**
 * \file ModuleLoaderUTest.cpp
 *
 *  \date Dec 7, 2012
 *  \author Alexandre Tuleu
 */

#include "ModuleLoaderUTest.h"
#include <robo-xeno/rtio/MockModule.h>
#include <robo-xeno/rtio/MockQueue.h>
#include <robo-xeno/rtio/MockIO.h>

#include "ModuleLoader.h"

#include <gmock/gmock-actions.h>

using ::testing::_;

namespace rx {

	void ModuleLoaderUTest::SetUp(){
		d_section = new ModulesSection(options::Section::Ptr(new options::Section("modules","")));
	}

	void ModuleLoaderUTest::TearDown(){
		delete d_section;
	}

	ModulesSection & ModuleLoaderUTest::Config(){ return *d_section; }

	void ModuleLoaderUTest::MockInitIO(rtio::Manager & manager){
		using namespace rtio;
		rtio::Queue::Ptr q = manager.AddQueue<MockQueue>();
		q->RegisterInput("a",rtio::Input::Ptr(new MockInput()));
		q->RegisterOutput("a",rtio::Output::Ptr(new MockOutput()));
	}


	void ModuleLoaderUTest::AddModule(options::Option<std::string>::Ptr o,
	                                  const std::string & name){
		o->SetValue(CMAKE_CURRENT_BINARY_DIR + std::string("/") + name + ";" );
	}


	TEST_F(ModuleLoaderUTest,LoadCriticalModule){
		AddModule(Config().CriticalOption(),"robo-xeno-module");
		rtio::MockModule * m = new rtio::MockModule();
		rtio::MockModule::SetLastCreated(m);

		EXPECT_CALL(*m,InitIO(_)).WillOnce(::testing::Invoke(MockInitIO));
		//we check that the module will be effectively destroy
		// (module is unloaded !)
		EXPECT_CALL(*m,Die());
		ModuleLoader ml;
		ml.LoadModuleFromConfig(Config());

		rtio::Manager manager;

		ml.StartQueuesOfModules(manager);


	}

	TEST_F(ModuleLoaderUTest,UnfoundCriticalModuleThrowException){
		AddModule(Config().CriticalOption(),"not-a-module");
		ModuleLoader ml;
		EXPECT_THROW({
			ml.LoadModuleFromConfig(Config());
		}, std::runtime_error);
	}

	TEST_F(ModuleLoaderUTest,UnfoundOptionnalModuleOk){
		AddModule(Config().OptionnalOption(),"not-a-module");
		ModuleLoader ml;
		EXPECT_NO_THROW({ml.LoadModuleFromConfig(Config());});

	}

} /* namespace rtio */
