/**
 * \file Process.h
 *
 *  \date Dec 7, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_PROCESS_H_
#define ROBO_XENO_PROCESS_H_


#include <tr1/memory>

#include <string>
#include <map>
#include <vector>

#include "IO.h"

//#include <native/mutex.h>

/**
 *
 */
namespace rx{
/**
 *
 */
class Process {
public:
	typedef std::tr1::shared_ptr<Process> Ptr;
	struct ControlInput {
		double value;
	private :
		Input::Ptr input;
		friend class Process;
	};
	typedef std::map<std::string,ControlInput> ControlInputByName;

	Process();
	virtual ~Process();

	ControlInputByName & ControlInputs();

	virtual rx::Input::Ptr  GetInput(const std::string & name)  = 0;
	virtual rx::Output::Ptr GetOutput(const std::string & name) = 0;


	virtual void Step(double ts) = 0;

	//thread safe methods
	void BufferedWrite(ControlInput & in);
	//Not implemented yet !!!!
	//double BufferedRead(rx::Output::Ptr & i);

	void Flush();
protected :
	void AddControlInput(const std::string & name,
	                     const Input::Ptr & i,
	                     double initialVlaue);
private :
	typedef std::vector<ControlInput> InputBuffer;
	InputBuffer d_inputBuffer;
	ControlInputByName d_controlInputs;


//	RT_MUTEX d_mutex;



};
}
#endif // ROBO_XENO_PROCESS_H_
