/**
 * \file ProcessUtest.h
 *
 *  \date Dec 7, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_PROCESSUTEST_H_
#define ROBO_XENO_PROCESSUTEST_H_

#include <gmock/gmock.h>
#include <gtest/gtest.h>

#include "IO.h"
#include "Process.h"

/**
 *
 */
namespace rx{

	class MockInput : public Input {
	public :
		typedef std::tr1::shared_ptr<MockInput> Ptr;
		virtual ~MockInput();
		MOCK_METHOD1(SendToProcess,void(double));
	};

	class MockOutput : public Output {
	public :
		typedef std::tr1::shared_ptr<MockOutput> Ptr;
		virtual ~MockOutput();
		MOCK_METHOD0(ReadFromProcess,double());
	};

	class MockProcess : public Process {
	public :
		MockProcess();
		virtual ~MockProcess();

		MOCK_METHOD1(GetInput,rx::Input::Ptr(const std::string &));
		MOCK_METHOD1(GetOutput,rx::Output::Ptr(const std::string &));
		MOCK_METHOD1(Step,void(double));

		const MockInput::Ptr & MyInput();
		const MockOutput::Ptr & MyOutput();

	private :
		MockInput::Ptr  d_input;
		MockOutput::Ptr d_output;

	};

/**
 *
 */
class ProcessUTest : public ::testing::Test {
protected :
		void SetUp();
		void TearDown();

		MockProcess d_process;
};
}
#endif // ROBO_XENO_PROCESSUTEST_H_
