/*
 * FileLogQueue.h
 *
 *  Created on: Dec 20, 2012
 *      Author: tuleu
 */

#ifndef FILELOGQUEUE_H_
#define FILELOGQUEUE_H_

#include "LogQueue.h"
#include <fstream>
namespace rtio {

class DataIO {
public :
	DataIO(const std::string & filename);
	virtual ~DataIO();

	void WriteHeader(const LogQueue::ListOfName & inputName,
	                 const LogQueue::ListOfName & outputName);

	void WriteRow(double time,
	              const DataVector & inputs,
	              const DataVector & outputs);

	void GetHeader(LogQueue::ListOfName & inputName,
	               LogQueue::ListOfName & outputName);

	bool GetRow(double & time,
	            DataVector & inputs,
	            DataVector & outputs);
private :
	typedef uint64_t FormatVersion;
	typedef std::size_t FieldLength;
	const static FormatVersion CURRENT_VERSION;

	void PrintListOfString(const LogQueue::ListOfName & list);
	void ReadListOfString(LogQueue::ListOfName & list);


	std::fstream  d_file;
	FieldLength d_inputSize,d_outputSize;
};



class FileLogQueue : public LogQueue {
public:
	FileLogQueue(ID id);
	virtual ~FileLogQueue();


	void SetFileName(const std::string & filename);
	void Init();
	void Process();
private :
	std::tr1::shared_ptr<DataIO> d_file;
};

} /* namespace rtio */
#endif /* FILELOGQUEUE_H_ */
