/*
 * Copyright (c) 2012 Alexandre Tuleu
 *
 * This file is part of xeno-cpg.
 *
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file IO.cpp
 *
 *  \date Jun 14, 2012
 *  \author Alexandre Tuleu
 */

#include "IO.h"

/**
 * \brief I/O operation namespace.
 *
 * These namespace holds all the main object to abstract I/O operation for your
 * platform.
 */
namespace rtio {

	IOBase::IOBase()
		: d_value(new double())
		, d_destroy(true){
	}

	IOBase::~IOBase(){
		if(d_destroy){
			delete d_value;
		}
	}

	double IOBase::Value() const { return * d_value;}

	void IOBase::SetValue(double value) { *d_value = value;}

	void IOBase::SetValuePtr(const DoublePtr & ptr,
	                         bool copyValue) {
		if(copyValue){
			*ptr = *d_value;
		}
		if(d_destroy){
			delete d_value;
		}
		d_destroy = false;
		d_value = ptr;
	}

	Input::Input() : d_processInput(){}

	Input::~Input() {}

	Output::Output() : d_processOutput(){}

	Output::~Output() {}

	void Input::SetProcessInput(const rx::Input::Ptr i){
		d_processInput = i;
	}

	bool Input::IsMapped() const{
		return d_processInput;
	}

	void Input::SendToProcess(){
		d_processInput->SendToProcess(Value());
	}

	void Output::SetProcessOutput(const rx::Output::Ptr & o){
		d_processOutput = o;
	}

	void Output::ReadFromProcess(){
		SetValue(d_processOutput->ReadFromProcess());
	}

	bool Output::IsMapped() const {
		return d_processOutput;
	}
}
