/*
 * Copyright (c) 2012 Alexandre Tuleu
 *
 * This file is part of xeno-cpg.
 *
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file RTIO.h
 *
 *  \date Jun 14, 2012
 *  \author Alexandre Tuleu
 */

#ifndef XENO_CPG_RTNODEIO_H_
#define XENO_CPG_RTNODEIO_H_

#include <tr1/memory>

#include <robo-xeno/process/IO.h>

namespace rtio {

class IOBase {
public :
	typedef double * DoublePtr;
	IOBase();
	virtual ~IOBase();

	double Value() const;
	void SetValue(double value);
	void SetValuePtr(const DoublePtr & ptr,
	                 bool copyValue);
private :


	DoublePtr  d_value;
	bool       d_destroy;
};


class Input : public IOBase {
public :
	typedef std::tr1::shared_ptr<rtio::Input> Ptr;
	Input();
	virtual ~Input();

	virtual void Read() = 0;

	void SetProcessInput(const rx::Input::Ptr i);
	void SendToProcess();
	bool IsMapped() const;
private :
	rx::Input::Ptr d_processInput;

};

class Output : public IOBase {
public :
	typedef std::tr1::shared_ptr<rtio::Output> Ptr;
	Output();
	virtual ~Output();

	virtual void Send() = 0;

	void SetProcessOutput(const rx::Output::Ptr & o);
	void ReadFromProcess();
	bool IsMapped() const;
private :
	rx::Output::Ptr d_processOutput;
};


} //namespace rt

/**
 *
 */


#endif // XENO_CPG_RTNODEIO_H_
