/**
 * \file LogQueue.cpp
 *
 *  \date Dec 13, 2012
 *  \author Alexandre Tuleu
 */

#include "LogQueue.h"
#include "Manager.h"

#include <robo-xeno/NativeError.h>
#include <iostream>

namespace rtio {

	LogQueue::LogQueue(ID id)
		: d_mask((1UL << id))
		, d_task(new RT_TASK()){
		int res = rt_task_create(d_task.get(),NULL,0,0,0);
		if(res){
			throw rx::NativeError("COuld not create task",-res);
		}
	}

	LogQueue::~LogQueue(){
	}


	void LogQueue::StartTask(){
		//ok reinterpreted_cast will be reinterpreted again.
		int res = rt_task_start(d_task.get(),&LogQueue::TaskBody,reinterpret_cast<void*>(this));
		if(res){
			throw rx::NativeError("Could ot start task",-res);
		}
	}

	void LogQueue::StopTask(){
		d_task = NativeHolder<RT_TASK>();
	}

	void LogQueue::TaskBody(void * itself){
		//we are private, we know itself is already reinterpreted_casted
		reinterpret_cast<LogQueue*>(itself)->TaskBody();
	}

	void LogQueue::SetTime(double t){
		d_time = t;
	}

	void LogQueue::SetName(const IOHolder<Input>::NameByIndex& inputNames ,
	                       const IOHolder<Output>::NameByIndex& outputNames){
		d_inputNames = inputNames;
		d_outputNames = outputNames;
		d_inData.assign(d_inputNames.size(),0);
		d_outData.assign(d_outputNames.size(),0);
	}

	void LogQueue::SetData(const DataVector& input , const DataVector& output){
		d_inData = input;
		d_outData = output;
	}

	double LogQueue::Time(){
		return d_time;
	}

	const LogQueue::ListOfName& LogQueue::InputNames() const{
		return d_inputNames;
	}

	const LogQueue::ListOfName& LogQueue::OutputNames() const{
		return d_outputNames;
	}

	const DataVector& LogQueue::InputValues() const{
		return d_inData;
	}

	const DataVector& LogQueue::OutputValues() const{
		return d_outData;
	}

	void LogQueue::TaskBody(){
		RT_EVENT event;
		int res = rt_event_bind(&event,rtio::Manager::LogEventName,TM_INFINITE);
		if(res){
			throw rx::NativeError("Could not bind event",-res);
		}
		unsigned long r_mask;
		while(true){
			res = rt_event_wait(&event,d_mask,&r_mask,EV_ANY,TM_INFINITE);
			rt_task_set_mode(T_PRIMARY,0,NULL);
			if(res){
				std::cerr << rx::NativeError("Could not wait event",-res).what() << std::endl;
				break;
			}

			this->Process();

			rt_event_clear(&event,d_mask,&r_mask);
		}
	}
} /* namespace rx */
