/**
 * \file LogQueue.h
 *
 *  \date Dec 13, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_LOGQUEUE_H_
#define ROBO_XENO_LOGQUEUE_H_

#include "IOHolder.h"
#include <robo-xeno/NativeHolder.h>
#include <tr1/memory>
/**
 *
 */
namespace rtio {
/**
 *
 */
class LogQueue {
public:
	typedef unsigned int ID;
	typedef std::vector<std::string> ListOfName;
	typedef std::tr1::shared_ptr<LogQueue> Ptr;
	LogQueue(ID id);
	virtual ~LogQueue();

	void SetTime(double t);
	void SetName(const IOHolder<Input>::NameByIndex & inputNames,
	             const IOHolder<Output>::NameByIndex & outputNames);
	void SetData(const DataVector & input,
	             const DataVector & output);

	virtual void Init()    = 0;
	virtual void Process() = 0;
	void StartTask();
	void StopTask();

	double Time();
	const ListOfName & InputNames() const;
	const ListOfName & OutputNames() const;

	const DataVector & InputValues() const;
	const DataVector & OutputValues() const;

private :
	static void TaskBody(void *itself);
	void TaskBody();


	unsigned long d_mask;
	NativeHolder<RT_TASK> d_task;
	ListOfName            d_inputNames,d_outputNames;
	DataVector            d_inData,d_outData;
	double                d_time;
};

}
#endif // ROBO_XENO_LOGQUEUE_H_
