/**
 * \file RTQueue.h
 *
 *  \date Dec 4, 2012
 *  \author Alexandre Tuleu
 */

#ifndef ROBO_XENO_RTQUEUE_H_
#define ROBO_XENO_RTQUEUE_H_

#include <tr1/memory>
#include <map>
#include <string>

#include <native/task.h>

#include <biorob-cpp/base/common.h>

#include <robo-xeno/NativeHolder.h>

#include <robo-xeno/rtio/IO.h>

namespace rtio {

class Manager;

class Queue {
		DISABLE_COPY(Queue)
		DISABLE_ASSIGNEMENT(Queue)
	public :
		typedef std::size_t ID;
		typedef std::tr1::shared_ptr<Queue> Ptr;
		typedef std::map<std::string,Output::Ptr> OutputByName;
		typedef std::map<std::string,Input::Ptr>  InputByName;


		enum Mode {
			Realtime    = 0,
			NonRealtime = 1
		};

		Queue (Manager & manager,Queue::ID id,int priority, Mode m = Realtime);
		virtual ~Queue();

		Manager & GetManager();

		void RegisterInput(const std::string & name,Input::Ptr input);
		void RegisterOutput(const std::string & name,Output::Ptr output);


		const OutputByName & Outputs() const;
		const InputByName & Inputs() const;


		virtual void PreSend()  = 0;
		virtual void PreRead()  = 0;
		virtual void PostRead() = 0;

		void StartTask();
		void StopTask();

		void Loop();

	private :
		static void RtBody(void * itself);
		void DeleteTask();

		rtio::Manager &  d_manager;
		NativeHolder<RT_TASK> d_task;
		int      d_priority;
		Mode     d_mode;
		const ID d_id;

		InputByName  d_inputs;
		OutputByName d_outputs;
	};
}
#endif // ROBO_XENO_RTQUEUE_H_
