/**
 * \file QueueUTest.cpp
 *
 *  \date Dec 4, 2012
 *  \author Alexandre Tuleu
 */

#include "QueueUTest.h"
#include "MockQueue.h"
#include "MockIO.h"

#include <stdexcept>

namespace rtio {

void QueueUTest::SetUp(){
	d_queue = d_manager.AddQueue<rtio::MockQueue>();
	d_queue->RegisterInput("a",Input::Ptr(new MockInput()));
	d_queue->RegisterInput("b",Input::Ptr(new MockInput()));

	d_queue->RegisterOutput("a",Output::Ptr(new MockOutput()));
	d_queue->RegisterOutput("b",Output::Ptr(new MockOutput()));
}


void QueueUTest::TearDown(){

}

bool QueueUTest::ContainsInput(const std::string & name){
	return d_queue->Inputs().find(name) != d_queue->Inputs().end();
}

bool QueueUTest::ContainsOutput(const std::string & name){
	return d_queue->Outputs().find(name) != d_queue->Outputs().end();
}

TEST_F(QueueUTest,UniqueInputName){
	EXPECT_THROW({
		d_queue->RegisterInput("a", Input::Ptr(new MockInput()));
	}, std::runtime_error);
}

TEST_F(QueueUTest,UniqueOutputName){
	EXPECT_THROW({
		d_queue->RegisterOutput("a", Output::Ptr(new MockOutput()));
	}, std::runtime_error);
}


TEST_F(QueueUTest,ConserveContent){
	EXPECT_TRUE(ContainsInput("a"));
	EXPECT_TRUE(ContainsInput("b"));
	EXPECT_FALSE(ContainsInput("c"));

	EXPECT_TRUE(ContainsOutput("a"));
	EXPECT_TRUE(ContainsOutput("b"));
	EXPECT_FALSE(ContainsOutput("c"));
}

}
