

#include <robo-xeno/rtio/FileLogQueue.h>

#include <boost/program_options.hpp>

#include <iomanip>

std::string ClampString(const std::string & s, int size){
	if(s.size() < size){
		return s;
	} else {
		return s.substr(0,size - 3) + "...";
	}
}

int main(int argc, char ** argv){
	namespace po= boost::program_options;
	typedef rtio::LogQueue::ListOfName ListOfName;
	po::variables_map  vm;
	po::options_description desc;


	desc.add_options()
		("help,h","produce help message")
		("input,i",po::value<std::string>(),"input file")
		("output,o",po::value<std::string>(),"output file")
		("precision,p",po::value<int>(),"precision for the field");


	po::store(po::parse_command_line(argc,argv,desc),vm);
	po::notify(vm);

	if(!vm.count("input")){
		throw std::runtime_error("You need to provide an input file");
	}
	int precision(5);
	if(!vm.count("precision")){
		precision = std::max(10,vm["precision"].as<int>());
	}


	std::string input = vm["input"].as<std::string>();

	std::string output;
	if(vm.count("output")){
		output = vm["output"].as<std::string>();
	} else {
		output = input + ".txt";
	}

	if(output == input){
		throw std::runtime_error("Output and input file need to be different");
	}

	rtio::DataIO inFile(input);

	std::ofstream outFile(output.c_str());

	ListOfName inputNames,outputNames;
	rtio::DataVector inputValues,outputValues;
	double time;

	inFile.GetHeader(inputNames,outputNames);

	outFile << std::setw(precision+7) << ClampString("Time",precision+7);

	for(ListOfName::const_iterator n = inputNames.begin();
	    n != inputNames.end();
	    ++n){
		outFile << " " << std::setw(precision+7) << ClampString(*n,precision+7);
	}

	for(ListOfName::const_iterator n = outputNames.begin();
	    n != outputNames.end();
	    ++n){
		outFile << " " << std::setw(precision+7) << ClampString(*n,precision+7);
	}
	outFile << std::endl;

	while(inFile.GetRow(time,inputValues,outputValues)){
		outFile << std::setw(precision) << std::scientific << std::setprecision(precision)  << time;


		for(rtio::DataVector::const_iterator v = inputValues.begin();
		    v != inputValues.end();
		    ++v){
			outFile << " " <<  std::setw(precision) << std::scientific << std::setprecision(precision)  << *v;
		}

		for(rtio::DataVector::const_iterator v = outputValues.begin();
		    v != outputValues.end();
		    ++v){
			outFile << " " <<  std::setw(precision) << std::scientific << std::setprecision(precision)  << *v;
		}

		outFile << std::endl;
	}

	return 0;
}
