/* 
 * Copyright (c) 2012 Alexandre Tuleu
 * 
 * This file is part of xeno-cpg.
 * 
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 * 
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file TaskRegister.cpp
 *
 *  \date Jun 6, 2012
 *  \author Alexandre Tuleu
 */

#include "TaskRegister.h"
#include <pthread.h>

#include <map>

class TaskRegister {

	friend void task_register_start_register_new_task();

	friend void task_register_end_register_new_task(pthread_t thread, RT_TASK * task);

	friend RT_TASK * rt_task_self();
private :
	typedef std::map<pthread_t,RT_TASK *> TaskByThread;
	static pthread_mutex_t s_mutex;
	static TaskByThread s_tasks;
};

pthread_mutex_t TaskRegister::s_mutex = PTHREAD_MUTEX_INITIALIZER;
TaskRegister::TaskByThread TaskRegister::s_tasks;

#ifdef __cplusplus
extern "C" {
#endif



void task_register_start_register_new_task(){

	pthread_mutex_lock(&TaskRegister::s_mutex);

}

void task_register_end_register_new_task(pthread_t thread, RT_TASK * task){
	TaskRegister::TaskByThread::const_iterator fi = TaskRegister::s_tasks.find(thread);

	if(fi != TaskRegister::s_tasks.end()){
		pthread_mutex_unlock(&(TaskRegister::s_mutex));
		return;
	}
	TaskRegister::s_tasks.insert(std::make_pair(thread,task));

	pthread_mutex_unlock(&(TaskRegister::s_mutex));
}

RT_TASK * rt_task_self(){
	RT_TASK * res(NULL);
	pthread_mutex_lock(&(TaskRegister::s_mutex));
	pthread_t self = pthread_self();

	TaskRegister::TaskByThread::const_iterator fi = TaskRegister::s_tasks.find(self);

	if(fi != TaskRegister::s_tasks.end()){
		res = fi->second;
	}

	pthread_mutex_unlock(&(TaskRegister::s_mutex));
	return res;
}

#ifdef __cplusplus
}
#endif
