/*
 * Copyright (c) 2012 Alexandre Tuleu
 *
 * This file is part of xeno-cpg.
 *
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "mutex.h"


#include <map>
#include <string>
#include <cstdlib>

class MutexRegister{
public :
	typedef std::map<std::string,pthread_mutex_t *> MutexByName;
	static pthread_mutex_t * Find(const std::string & name);
	static pthread_mutex_t * Create(const std::string & name);
	static pthread_mutex_t s_mutex;
	static bool s_init;
	static MutexByName s_mutexes;
};


pthread_mutex_t MutexRegister::s_mutex = PTHREAD_MUTEX_INITIALIZER;
bool MutexRegister::s_init(false);
MutexRegister::MutexByName MutexRegister::s_mutexes;


pthread_mutex_t * MutexRegister::Find(const std::string & name){

	pthread_mutex_lock(&s_mutex);
	pthread_mutex_t * res(NULL);
	MutexByName::iterator fi = s_mutexes.find(name);
	if(fi != s_mutexes.end()){
		res = fi->second;
	}
	pthread_mutex_unlock(&s_mutex);

	return res;

}
pthread_mutex_t * MutexRegister::Create(const std::string & name){
	pthread_mutex_lock(&s_mutex);
	pthread_mutex_t * res(NULL);
	MutexByName::iterator fi = s_mutexes.find(name);
	if(fi != s_mutexes.end()){
		res = fi->second;
	} else {
		res = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
		pthread_mutex_init(res,NULL);
		s_mutexes.insert(std::make_pair(name,res));
	}
	pthread_mutex_unlock(&s_mutex);

	return res;
}


#ifdef __cplusplus
extern "C" {
#endif


int rt_mutex_create (RT_MUTEX *mutex, const char *name){
	if(name == NULL){
		mutex->mutex = (pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
		pthread_mutex_init(mutex->mutex,NULL);
		return 0;
	}
	mutex->mutex = (MutexRegister::Find(name));
	if(mutex->mutex == NULL){
		mutex->mutex = MutexRegister::Create(name);
	}
	return 0;
}

int rt_mutex_delete(RT_MUTEX * mutex){
	pthread_mutex_destroy(mutex->mutex);
	free(mutex->mutex);
	return 0;
}

int rt_mutex_acquire (RT_MUTEX *mutex, RTIME timeout){
	return pthread_mutex_lock(mutex->mutex);

}

int rt_mutex_release (RT_MUTEX *mutex){
	return pthread_mutex_unlock(mutex->mutex);
}

int rt_mutex_bind (RT_MUTEX *mutex, const char *name, RTIME timeout){
	pthread_mutex_t * m(NULL);
	while(true){
		m = MutexRegister::Find(name);
		if( m!= NULL ){
			mutex->mutex = m;
			return 0;
		}
		usleep(10000);
	}
	return 0;
}

int rt_mutex_unbind (RT_MUTEX *mutex){
	mutex->mutex = NULL;
	return 0;
}

#ifdef __cplusplus
}
#endif
