/**
 * \file pipe.cpp
 *
 *  \date Dec 7, 2012
 *  \author Alexandre Tuleu
 */



#include "pipe.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <string>

extern "C" {

	int rt_pipe_create (RT_PIPE *pipe,
	                    const char *name,
	                    int minor,
	                    size_t poolsize){
		pipe->name = tempnam("/tmp","xeno-emulation-pipe");
		if(!pipe->name){
			return -1;
		}
		if(mkfifo(pipe->name,00644)){
			return -2;
		}
		pipe->file = open(pipe->name,O_APPEND);
		if(pipe->file == -1){
			return -3;
		}
		if(name){
			std::string symName(XENO_EMULATION_PIPE_PATH);
			symName += name;
			if(symlink(pipe->name,symName.c_str())){
				return -4;
			}
		}
		return 0;
	}

	int rt_pipe_delete (RT_PIPE *pipe){
		if(pipe->file != -1){
			close(pipe->file);
		}
		return 0;
	}

	ssize_t rt_pipe_read (RT_PIPE *pipe, void *buf, size_t size, RTIME timeout){
		return read(pipe->file,buf,size);
	}

	ssize_t rt_pipe_write (RT_PIPE *pipe, const void *buf, size_t size, int mode){
		return write(pipe->file,buf,size);
	}



}
