/**
 * \file sem.cpp
 *
 *  \date Dec 5, 2012
 *  \author Alexandre Tuleu
 */


#include "sem.h"

#include <map>
#include <string>
#include <cstring>
#include <cstdlib>

typedef std::map<std::string,RT_SEM> SemByName;
SemByName all_sems;
extern "C" {

	int rt_sem_create (RT_SEM *sem, const char *name, unsigned long icount, int mode){
		if(name){
			if(all_sems.find(name) != all_sems.end()){
				return -1;
			}
		}

		sem->s = (sem_t*)malloc(sizeof(sem_t));
		if(name){
			sem->name = (char *)malloc(strlen(name) + 1);
			strcpy(sem->name,name);
			RT_SEM saved;
			saved.s = sem->s;
			saved.name = sem->name;
			all_sems.insert(std::make_pair(name,saved));
		} else {
			sem->name = 0;
		}

		sem_init(sem->s,0,icount);

		return 0;
	}

	int rt_sem_delete (RT_SEM *sem){
		if(sem->name){
			SemByName::iterator sem_s = all_sems.find(sem->name);
			if(sem_s != all_sems.end()){
				all_sems.erase(sem_s);
			}
			free(sem->name);
		}
		sem_destroy(sem->s);
		free(sem->s);
		return 0;
	}

	int rt_sem_p (RT_SEM *sem, RTIME timeout){
		if(timeout == TM_INFINITE){
			sem_wait(sem->s);
			return 0;
		}
		unsigned long nb_usec(0);
		while(nb_usec <= timeout/1000){
			int res = sem_trywait(sem->s);
			if(res == 0){
				return 0;
			}
			usleep(1000);
			nb_usec += 1000;
		}
		return -1;
	}


	int rt_sem_v (RT_SEM *sem){
		return sem_post(sem->s);

	}

	int rt_sem_inquire(RT_SEM * sem, RT_SEM_INFO * info){
		int c;
		sem_getvalue(sem->s,&c);
		info->count = c;
		return 0;
	}

}
