#include <stdint.h>
#include <unistd.h>

#include "task.h"
#include "TaskRegister.h"
#include "timer.h"

int 	rt_task_create (RT_TASK *task, const char *name, int stksize, int prio, int mode){
	task->timer = NULL;
	pthread_attr_t * a = &(task->attr);
	pthread_attr_init(a);
	if( mode & T_JOINABLE ) {
		pthread_attr_setdetachstate(a,PTHREAD_CREATE_JOINABLE);
	}
	if(stksize != 0){
		pthread_attr_setstacksize(a,stksize);
	}
	task->thread = 0;
	return 0;
}

int 	rt_task_inquire (RT_TASK *task, RT_TASK_INFO *info){
	return pthread_getschedparam(task->thread,&(info->policy),&(info->param));
}

int 	rt_task_delete (RT_TASK *task){

	pthread_attr_destroy(&(task->attr));
	if(task->thread != 0){
		pthread_detach(task->thread);
	}
	return 0;
}

int 	rt_task_start (RT_TASK *task, void (*entry)(void*), void * cookie){
	typedef void*(*good_fct_ptr)(void*);
	pthread_create( &(task->thread) , &(task->attr) , (good_fct_ptr)entry , cookie);
	return 0;
}

int 	rt_task_set_periodic (RT_TASK *task, RTIME idate, RTIME period){
	task->timer = create_robo_xeno_timer();
	return robo_xeno_timer_set_period(task->timer,period);

}

int 	rt_task_join (RT_TASK *task){
	void * status;
	pthread_join(task->thread ,&status);
	return 0;
}

int 	rt_task_wait_period (unsigned long *overruns_r){
	RT_TASK * self = rt_task_self();

	if (self == NULL){
		return -EPERM;
	}

	if(self->timer == 0){
		return -EWOULDBLOCK;
	}


	return robo_xeno_timer_wait_period(self->timer,overruns_r);



}

int 	rt_task_set_mode (int clrmask, int setmask, int *mode_r){
	return 0;
}

int 	rt_task_sleep (RTIME delay){
	usleep(delay/1e3);
	return 0;
}


int rt_task_sleep_until(RTIME delay){
	delay -= rt_timer_read();
	usleep(delay/1e3);
	return 0;
}

int 	rt_task_shadow(RT_TASK * task, const char * name,int prio, int mode){
	//do nothing
	return 0;
}
