/*
 * Copyright (c) 2012 Alexandre Tuleu
 *
 * This file is part of xeno-cpg.
 *
 *   xeno-cpg is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   xeno-cpg is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef ROBO_XENO_EMULATION_NATIVE_TASK_H_
#define ROBO_XENO_EMULATION_NATIVE_TASK_H_

#include "types.h"

#include <pthread.h>

#include <xeno-emulation/timer.h>

#ifdef __cplusplus
extern "C" {
#endif
	enum TaskModeFlags {
		T_PRIMARY,
		T_WARNSWC,
		T_FPU
	};

	struct rt_task {
		pthread_t thread;
		pthread_attr_t attr;
		robo_xeno_timer * timer;
	};

	typedef struct rt_task RT_TASK;

	struct rt_task_info {
		int policy;
		struct sched_param param;
	};

	typedef struct rt_task_info RT_TASK_INFO;


	int 	rt_task_create (RT_TASK *task, const char *name, int stksize, int prio, int mode);

	int 	rt_task_inquire (RT_TASK *task, RT_TASK_INFO *info);

	int 	rt_task_delete (RT_TASK *task);


	int 	rt_task_start (RT_TASK *task, void (* entry)(void *), void *cookie);

	int 	rt_task_set_periodic (RT_TASK *task, RTIME idate, RTIME period);

	int 	rt_task_join (RT_TASK *task);

	int 	rt_task_wait_period (unsigned long *overruns_r);

	int 	rt_task_set_mode (int clrmask, int setmask, int *mode_r);

	int 	rt_task_sleep (RTIME delay);

	int 	rt_task_sleep_until (RTIME delay);

	int 	rt_task_shadow(RT_TASK * task, const char * name,int prio, int mode);

#ifdef __cplusplus
}
#endif


#endif //ROBO_XENO_EMULATION_NATIVE_TASK_H_
